@@ -1,5 +1,3 @@
-/* $Xorg: dirfile.c,v 1.4 2001/02/09 02:04:03 xorgcvs Exp $ */
-
 /*
 
 Copyright 1991, 1998  The Open Group
@@ -25,7 +23,6 @@ used in advertising or otherwise to promote the sale, use or other dealings
 in this Software without prior written authorization from The Open Group.
 
 */
-/* $XFree86: xc/lib/font/fontfile/dirfile.c,v 3.17 2004/02/08 01:52:27 dawes Exp $ */
 
 /*
  * Author:  Keith Packard, MIT X Consortium
@@ -45,6 +42,7 @@ in this Software without prior written authorization from The Open Group.
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <errno.h>
+#include <limits.h>
 
 static Bool AddFileNameAliases ( FontDirectoryPtr dir );
 static int ReadFontAlias ( char *directory, Bool isFile,
@@ -66,6 +64,9 @@ FontFileReadDirectory (char *directory, FontDirectoryPtr *pdir)
                 status;
     struct stat	statb;
     static char format[24] = "";
+#if defined(WIN32)
+    int i;
+#endif
 
     FontDirectoryPtr	dir = NullFontDirectory;
 
@@ -73,7 +74,7 @@ FontFileReadDirectory (char *directory, FontDirectoryPtr *pdir)
 	return BadFontPath;
 
     /* Check for font directory attributes */
-#if !defined(__UNIXOS2__) && !defined(WIN32)
+#if !defined(WIN32)
     if ((ptr = strchr(directory, ':'))) {
 #else
     /* OS/2 and WIN32 path might start with a drive letter, don't clip this */
@@ -90,7 +91,7 @@ FontFileReadDirectory (char *directory, FontDirectoryPtr *pdir)
     strcat(dir_file, FontDirFile);
     file = fopen(dir_file, "rt");
     if (file) {
-#ifndef WIN32        
+#ifndef WIN32
 	if (fstat (fileno(file), &statb) == -1)
 #else
 	if (stat (dir_file, &statb) == -1)
@@ -115,7 +116,7 @@ FontFileReadDirectory (char *directory, FontDirectoryPtr *pdir)
 		MAXFONTFILENAMELEN-1, MAXFONTNAMELEN-1);
 
 	while ((count = fscanf(file, format, file_name, font_name)) != EOF) {
-#if defined(__UNIXOS2__) || defined(WIN32)
+#if defined(WIN32)
 	    /* strip any existing trailing CR */
 	    for (i=0; i<strlen(font_name); i++) {
 		if (font_name[i]=='\r') font_name[i] = '\0';
@@ -135,7 +136,7 @@ FontFileReadDirectory (char *directory, FontDirectoryPtr *pdir)
 	    FontFileAddFontFile (dir, font_name, file_name);
 	}
 	fclose(file);
-	
+
     } else if (errno != ENOENT) {
 	return BadFontPath;
     }
@@ -173,6 +174,9 @@ FontFileDirectoryChanged(FontDirectoryPtr dir)
     }
     if (dir->dir_mtime != statb.st_mtime)
 	return TRUE;
+
+    if ((strlen(dir->directory) + sizeof(FontAliasFile)) > sizeof(dir_file))
+	return FALSE;
     strcpy (dir_file, dir->directory);
     strcat (dir_file, FontAliasFile);
     if (stat (dir_file, &statb) == -1)
@@ -185,7 +189,7 @@ FontFileDirectoryChanged(FontDirectoryPtr dir)
 	return TRUE;
     return FALSE;
 }
-    
+
 /*
  * Make each of the file names an automatic alias for each of the files.
  */
@@ -209,7 +213,7 @@ AddFileNameAliases(FontDirectoryPtr dir)
 	renderer = FontFileMatchRenderer (fileName);
 	if (!renderer)
 	    continue;
-	
+
 	len = strlen (fileName) - renderer->fileSuffixLen;
 	if (len >= sizeof(copy))
 	    continue;
@@ -371,8 +375,11 @@ lexAlias(FILE *file, char **lexToken)
 	    int         nsize;
 	    char       *nbuf;
 
+	    if (tokenSize >= (INT_MAX >> 2))
+		/* Stop before we overflow */
+		return EALLOC;
 	    nsize = tokenSize ? (tokenSize << 1) : 64;
-	    nbuf = (char *) xrealloc(tokenBuf, nsize);
+	    nbuf = realloc(tokenBuf, nsize);
 	    if (!nbuf)
 		return EALLOC;
 	    tokenBuf = nbuf;
